#include <M5Core2.h>
#include "opm.jpg.c"
#include "UNIT_ENV.h"

typedef struct __attribute__((__packed__)) {
  uint32_t timestamp;
  int16_t temperature;
  uint8_t humidity;
  uint32_t pressure;
} sensor_data_t;

#ifdef WITH_BUTTONS

#define BLUE_BUTTON_PIN 13
#define RED_BUTTON_PIN 14
#define DEBOUNCE_MS 10

Button RedButton = Button(RED_BUTTON_PIN, true, DEBOUNCE_MS);
Button BlueButton = Button(BLUE_BUTTON_PIN, true, DEBOUNCE_MS);

#endif

#ifdef WITH_ENVIII
SHT3X sht30;
QMP6988 qmp6988;

unsigned long last_ts = 0;

sensor_data_t sensor_data;
#endif

void setup() {
#ifndef WITH_ENVIII
  M5.begin();
#else
  M5.begin(true, false, true, true);
  qmp6988.init();
#endif
  
  // Affichage logo
  M5.Lcd.drawJpg(opm, sizeof(opm), 75, 30);  
  delay(1000);
  M5.Lcd.clear();

  M5.Lcd.setTextSize(3);

#ifdef WITH_BUTTONS
  M5.Lcd.drawString("Buttons", 100, 50);
#endif

#ifdef WITH_ENVIII
  M5.Lcd.drawString("ENVIII", 110, 150);
#endif
}

void loop() {

#ifdef WITH_BUTTONS
  M5.update();
  if (M5.BtnA.wasPressed()) {
    Serial.println("BtnA button pressed");
  }

  if (M5.BtnB.wasPressed()) {
    Serial.println("BtnB button pressed");
  }

  if (M5.BtnC.wasPressed()) {
    Serial.println("BtnC button pressed");
  }

  if (RedButton.wasPressed()) {
    Serial.println("Red button pressed");
  }

  if (BlueButton.wasPressed()) {
    Serial.println("Blue button pressed");
  }
#endif

#ifdef WITH_ENVIII
  long ts = millis();
  if (ts - last_ts > 1000) {
    last_ts = ts;

    // Acquisition
    float pressure = qmp6988.calcPressure();
    
    float tmp = 0, hum = 0;
    if (sht30.get() == 0) {
      tmp = sht30.cTemp;
      hum = sht30.humidity;
    }

    Serial.printf("T = %.1f C, H = %.0f %%, P = %.0f Pa\n", tmp, hum, pressure);

    // Encodage
    sensor_data = {
      .timestamp = (uint32_t) ts,
      .temperature = (int16_t) (tmp * 100),
      .humidity = (uint8_t) hum,
      .pressure = (uint32_t) pressure
    };

    // Affichage données
    for (int i = 0; i < sizeof(sensor_data); i++) {
      Serial.print(*((uint8_t *)&sensor_data + i), HEX);
      Serial.print(' ');
    }
    Serial.println();
    
  }
#endif

  delay(100); 
}